'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var fs = _interopDefault(require('fs-extra'));
var path = _interopDefault(require('path'));
var util = require('util');
var get = _interopDefault(require('lodash.get'));
var prompt = _interopDefault(require('prompt'));
var find = require('find');
var execa = _interopDefault(require('execa'));

/* eslint-env node */
/* eslint-disable no-console */
function usage() {
  console.log(`Initialize a new workflow app

Usage:
  npm init workflow-app [target]
  npx create-workflow-app [target]
  yarn create workflow-app [target]

Initializes the current directory or target if given
with the create-workflow-app initializer.
`);
}

function parse(args) {
  if (args.length < 1) {
    console.error('Invalid number of arguments');
    console.log();
    usage();
    process.exit(1);
  }

  const [node, cmd, targetFolder] = args; // eslint-disable-line

  if (args.length === 2) {
    return {};
  } else if (args.length === 3) {
    return { targetFolder };
  } else {
    console.error("Invalid arguments: '" + args.join(' ') + "'");
    console.log();
    usage();
    process.exit(1);
  }
}

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

const { targetFolder } = parse(process.argv);

Promise.resolve().then(promptPackageDetails).then(renderTemplates).then(copyStaticFiles).catch(err => console.error(err)); // eslint-disable-line no-console

async function promptPackageDetails() {
  const packageName = path.basename(resolveTarget());

  var schema = {
    properties: {
      packageName: {
        pattern: /^[a-zA-Z-]+$/,
        default: packageName,
        required: true
      },
      description: {
        required: false
      }
    }
  };

  prompt.start();

  return util.promisify(prompt.get)(schema);
}

function renderTemplates({ packageName, description }) {
  const versions = readVersions();
  const templates = findTemplates();

  for (let templateName of templates) {
    const pkgPath = resolveTarget(templateName);
    const templatePath = resolveSource('templates', templateName);

    renderTemplate(templatePath, pkgPath, _extends({}, versions, {
      packageName,
      description
    }));
  }
}

function copyStaticFiles() {
  const source = resolveSource('static/');
  const destination = resolveTarget();

  fs.copySync(source, destination);
}

function findTemplates() {
  const templateBase = resolveSource('templates');

  return find.fileSync(templateBase).map(absolutePath => path.relative(templateBase, absolutePath));
}

function readVersions() {
  if (!readVersions.versions) {
    const rootPkgPath = resolveRoot('package.json');
    const rootPkgString = fs.readFileSync(rootPkgPath, 'utf8');
    const rootPkg = JSON.parse(rootPkgString);
    const { version, devDependencies } = rootPkg;

    readVersions.versions = _extends({}, devDependencies, {
      'workflow.app.version': version
    });
  }

  return readVersions.versions;
}

async function renderTemplate(template, targetPath, values) {
  const templateString = fs.readFileSync(template, 'utf8');

  let renderedTemplate = templateString.replace(/{{([a-zA-Z.-]*)}}/g, (_, prelude) => get(values, prelude));

  if ((await isWorkflowRepo()) && isPackageJson(targetPath)) {
    renderedTemplate = filterWorkflowRepoPackageJson(renderedTemplate);
  }

  await fs.mkdirp(path.dirname(targetPath));
  fs.writeFileSync(targetPath, renderedTemplate);
}

function resolveRoot(...args) {
  return path.join(__dirname, '..', ...args);
}

function resolveSource(...args) {
  return path.join(__dirname, '..', 'files', ...args);
}

function resolveTarget(...args) {
  if (args) {
    return path.join(resolveTargetFolder(), ...args);
  } else {
    return resolveTargetFolder();
  }
}

function resolveTargetFolder() {
  if (targetFolder) {
    return path.resolve(targetFolder);
  } else {
    return path.resolve();
  }
}

function isPackageJson(path$$1) {
  return resolveTarget('package.json') === path$$1;
}

async function isWorkflowRepo() {
  try {
    const { stdout } = await execa('git', ['config', '--get', 'remote.origin.url']);
    return stdout.trim() === 'git@github.com:havardh/workflow.git';
  } catch (e) {
    return false;
  }
}

function filterWorkflowRepoPackageJson(pkgJsonString) {
  const json = JSON.parse(pkgJsonString);

  delete json.devDependencies;

  return JSON.stringify(json, null, 2) + '\n';
}
